from policyengine_it.model_api import *


class household_market_income(Variable):
    value_type = float
    entity = Household
    label = "Household market (pre-tax) income"
    unit = EUR
    definition_period = YEAR
    adds = ["total_individual_pre_tax_income"]



from policyengine_it.model_api import *


class low_income_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for low income tax credit"
    definition_period = YEAR

    def formula(person, period, parameters):
        lic_eligibility = parameters(
            period
        ).gov.ade.credits.low_income.eligibility
        employment_category = person("employment_category", period)

        lic_threshold = lic_eligibility[employment_category]
        eligible = (
            person("total_individual_pre_tax_income", period) <= lic_threshold
        )

        return eligible



from policyengine_it.model_api import *


class mario_besusso(Variable):
    value_type = float
    entity = Person
    label = "Total amount paid by Mario Besusso contributors"
    definition_period = YEAR

    def formula(person, period, parameters):
        is_eligible = person("mario_besusso_eligible", period)
        income = person("total_individual_pre_tax_income", period)

        p = parameters(period).private.mario_besusso.rates

        amount = p.calc(income)

        return is_eligible * amount



from policyengine_it.model_api import *


class social_security(Variable):
    value_type = float
    entity = Person
    label = "Total social security contribution"
    definition_period = YEAR

    def formula(person, period, parameters):
        employment_category = person("employment_category", period)
        category = employment_category.possible_values
        income = person("total_individual_pre_tax_income", period)

        p = parameters(period).gov.inps.social_security.rates

        social_security_amount = select(
            [
                employment_category == category.EMPLOYEE,
                employment_category == category.EXECUTIVE,
                employment_category == category.UNEMPLOYED,
            ],
            [
                p.employee.calc(income),
                p.executive.calc(income),
                p.unemployed.calc(income),
            ],
        )

        return social_security_amount



from policyengine_it.model_api import *


class fasi(Variable):
    # FASI has one base payment, plus optional additional; however,
    # our source material does not model the optional payments, hence leaving
    # this here for future expansion purposes
    value_type = float
    entity = Person
    label = "Total amount paid by Fondo Assistenza Sanitaria Industria (FASI) contributors"
    definition_period = YEAR
    adds = ["fasi_base_amount"]



from policyengine_it.model_api import *


class household_married(Variable):
    value_type = bool
    entity = Household
    definition_period = YEAR
    label = "Household head and spouse are married"

    def formula(household, period, parameters):
        person = household.members
        return household.any(person("is_spouse", period))



from policyengine_it.model_api import *


class household_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = Household
    label = "tax"
    documentation = "Total tax liability before refundable credits."
    unit = EUR
    definition_period = YEAR
    adds = ["income_tax_before_refundable_credits"]



from policyengine_it.model_api import *


class EmploymentCategory(Enum):
    # This is not defined well in the reference material
    EMPLOYEE = "Standard employee"
    EXECUTIVE = "Executive"
    UNEMPLOYED = "Unemployed"


class employment_category(Variable):
    value_type = Enum
    entity = Person
    label = "Employment category"
    possible_values = EmploymentCategory
    default_value = EmploymentCategory.EMPLOYEE
    definition_period = YEAR



from policyengine_it.model_api import *


class income_tax_before_credits(Variable):
    value_type = float
    entity = Person
    label = "Income tax before all tax credits"
    unit = EUR
    documentation = "Income tax before all tax credits"
    definition_period = YEAR

    def formula(person, period, parameters):
        income = person("total_individual_pre_tax_income", period)
        gov = parameters(period).gov.ade.tax.income
        return gov.income_tax_schedule.calc(income)



from policyengine_it.model_api import *


class shepherd_fund_base_amount(Variable):
    value_type = float
    entity = Person
    label = "Fixed, flat base amount paid by all Shepherd Fund contributors"
    definition_period = YEAR

    def formula(person, period, parameters):
        is_eligible = person("shepherd_fund_eligible", period)
        base_amount = parameters(period).private.shepherd_fund.base_amount

        return is_eligible * base_amount



from policyengine_it.model_api import *


class dependent_spouse_additional_amount(Variable):
    value_type = float
    entity = Household
    label = "Dependent spouse income tax credit additional amount"
    definition_period = YEAR

    def formula(household, period, parameters):
        p = parameters(period).gov.ade.credits.dependent_spouse

        is_eligible = household("dependent_spouse_eligible", period)
        income = household("household_market_income", period)
        additional_amount = p.additional.calc(income)

        return is_eligible * additional_amount



from policyengine_it.model_api import *


class dependent_spouse_eligible(Variable):
    value_type = bool
    entity = Household
    label = "Eligible for dependent spouse income tax credit"
    definition_period = YEAR

    def formula(household, period, parameters):
        p = parameters(period).gov.ade.credits.dependent_spouse.eligibility

        # Determine if household is married
        household_is_married: bool = household("household_married", period)

        # Pull overall household income pre-credits
        household_income: float = household("household_market_income", period)

        # Find spouse's income specifically
        spouse_income: float = household("spouse_market_income", period)

        # Find values for each eligibility criterium
        elig_income_dist: bool = household_income > spouse_income
        elig_spouse_income: bool = spouse_income <= p.spouse_income
        elig_household_income: bool = household_income <= p.household_income
        elig_marital_status: bool = household_is_married

        eligible = (
            elig_income_dist
            & elig_spouse_income
            & elig_household_income
            & elig_marital_status
        )

        return eligible



from policyengine_it.model_api import *


class household_net_income(Variable):
    value_type = float
    entity = Household
    label = "net income"
    unit = EUR
    definition_period = YEAR
    adds = ["household_market_income", "refundable_tax_credits"]
    subtracts = [
        "household_income_tax_before_refundable_credits",
        "insurance_and_pensions",
    ]



from policyengine_it.model_api import *

label = "Earnings"


class employment_income(Variable):
    # In a more built-out model, there would be multiple different
    # categories of income that all add together to form
    # total_individual_pre_tax_income. However, the existing model does not
    # differentiate, so we'll assume all income is employment_income
    value_type = float
    entity = Person
    label = "All income gained from standard employment"
    unit = EUR
    documentation = "Income from standard employment"
    definition_period = YEAR



from policyengine_it.model_api import *


class spouse_market_income(Variable):
    value_type = float
    entity = Household
    label = "Market income of dependent spouse within household"
    definition_period = YEAR

    def formula(household, period, parameters):
        person = household.members

        # Find spouse's income specifically
        is_spouse: bool = person("is_spouse", period)
        market_income = person("total_individual_pre_tax_income", period)

        spouse_market_income: float = np.where(
            is_spouse, person("total_individual_pre_tax_income", period), 0
        )
        spouse_market_income: float = household.sum(market_income * is_spouse)

        return spouse_market_income



from policyengine_it.model_api import *


class insurance_and_pensions(Variable):
    value_type = float
    entity = Person
    label = "insurance and pensions, total"
    unit = EUR
    definition_period = YEAR
    adds = [
        "social_security",
        "mario_negri",
        "mario_besusso",
        "shepherd_fund",
        "fasi",
    ]



from policyengine_it.model_api import *


class head_income(Variable):
    value_type = bool
    entity = Household
    label = "Income of head of household"
    definition_period = YEAR

    def formula(household, period, parameters):
        person = household.members

        # Find spouse's income specifically
        is_head: bool = person("is_head", period)
        head_income: float = (
            person("total_individual_pre_tax_income", period) * is_head
        )

        return head_income



from policyengine_it.model_api import *


class count_dependents(Variable):
    value_type = int
    entity = Household
    label = "Dependents"
    unit = EUR
    documentation = "Number of dependents"
    definition_period = YEAR
    adds = ["is_dependent"]



from policyengine_it.model_api import *


class mario_negri(Variable):
    value_type = float
    entity = Person
    label = "Total Mario Negri Fund payment for eligible contributors"
    definition_period = YEAR
    adds = ["mario_negri_base_amount", "mario_negri_variable_amount"]



from policyengine_it.model_api import *


class mario_negri_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Mario Negri Fund"
    definition_period = YEAR

    def formula(person, period, parameters):
        mn_eligible_groups = parameters(period).private.mario_negri.eligibility

        employment_category = person(
            "employment_category", period
        ).decode_to_str()[0]

        return employment_category in mn_eligible_groups



from policyengine_it.model_api import *


class non_refundable_tax_credits(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = EUR
    label = "Total value of national non-refundable tax credits"
    adds = "gov.ade.credits.non_refundable"



from policyengine_it.model_api import *

label = "Earnings"


class total_individual_pre_tax_income(Variable):
    # In a more built-out model, there would be different categories of income
    # that all form part of this variable. However, the existing model only contains
    # 'gross_income' (for head of household) and 'spouse income' (for spouse).
    # This will be used as the stand-in for that, even though it only includes one
    # subset, 'employment_income'.
    value_type = float
    entity = Person
    label = "Total individual pre-tax income"
    unit = EUR
    documentation = "Income from gainful employment"
    definition_period = YEAR
    adds = ["employment_income"]



from policyengine_it.model_api import *


class income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = Person
    definition_period = YEAR
    unit = EUR
    label = "Italian national income tax before refundable credits"
    documentation = "Income tax liability (including other taxes) after non-refundable credits are used, but before refundable credits are applied"

    def formula(person, period, parameters):
        income_tax_before_credits = person("income_tax_before_credits", period)
        non_refundable_tax_credits = person(
            "non_refundable_tax_credits", period
        )
        return max_(income_tax_before_credits - non_refundable_tax_credits, 0)



from policyengine_it.model_api import *


class individual_net_income(Variable):
    value_type = float
    entity = Person
    label = "Individual net income"
    unit = EUR
    definition_period = YEAR
    adds = ["total_individual_pre_tax_income"]
    subtracts = ["income_tax", "insurance_and_pensions"]



from policyengine_it.model_api import *


class shepherd_fund_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Shepherd Fund"
    definition_period = YEAR

    def formula(person, period, parameters):
        sf_eligible_groups = parameters(
            period
        ).private.shepherd_fund.eligibility

        employment_category = person(
            "employment_category", period
        ).decode_to_str()[0]

        return employment_category in sf_eligible_groups



from policyengine_it.model_api import *


class low_income_credit(Variable):
    value_type = float
    entity = Person
    label = "Value returned by the low income tax credit"
    definition_period = YEAR

    def formula(person, period, parameters):
        is_eligible = person("low_income_eligible", period)
        exemption_rate = parameters(
            period
        ).gov.ade.credits.low_income.exemption_rate
        income = person("total_individual_pre_tax_income", period)
        exempted_income = exemption_rate * income

        return is_eligible * exempted_income



from policyengine_it.model_api import *


class household_refundable_tax_credits(Variable):
    value_type = float
    entity = Household
    label = "refundable tax credits"
    definition_period = YEAR
    unit = EUR
    adds = ["refundable_tax_credits"]



from policyengine_it.model_api import *


class refundable_tax_credits(Variable):
    value_type = float
    entity = Person
    label = "refundable tax credits"
    unit = EUR
    definition_period = YEAR

    adds = "gov.ade.credits.refundable"



from policyengine_it.model_api import *


class dependent_spouse_base_amount(Variable):
    value_type = float
    entity = Household
    label = "Dependent spouse income tax credit base amount"
    definition_period = YEAR

    def formula(household, period, parameters):
        p = parameters(period).gov.ade.credits.dependent_spouse

        is_eligible = household("dependent_spouse_eligible", period)
        income = household("household_market_income", period)
        base_amount = p.base_amount.calc(income)

        phase_out_rate = p.phase_out.calc(income)
        prev_threshold = get_previous_threshold(income, p.phase_out.thresholds)
        phase_out_amount = phase_out_rate * (income - prev_threshold)

        return is_eligible * (base_amount - phase_out_amount)



from policyengine_it.model_api import *


class dependent_spouse_deduction(Variable):
    value_type = float
    entity = Person
    label = "Total value of the dependent spouse deduction"
    definition_period = YEAR

    adds = [
        "dependent_spouse_base_amount",
        "dependent_spouse_additional_amount",
    ]
